﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Windows.DataPresenter.ExcelExporter;
using System.Diagnostics;
using Infragistics.Windows.DataPresenter;

namespace ExportToExcelWithHeaderStyle
{
    using System.Collections.ObjectModel;
    using System.ComponentModel;
    using ExportToExcelWithHeaderStyle.Converters;
    using System.Threading;

    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            DataContext = Data.GetData(5, 2, 10);
        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {
            DataPresenterExcelExporter exporter = new DataPresenterExcelExporter();
            exporter.HeaderLabelExporting += new EventHandler<HeaderLabelExportingEventArgs>(exporter_HeaderLabelExporting);

            exporter.CellExporting += new EventHandler<CellExportingEventArgs>(exporter_CellExporting);

            exporter.Export(xamDataGrid1, "excel.xlsx", Infragistics.Documents.Excel.WorkbookFormat.Excel2007);

            Process.Start("excel.xlsx");
        }

        void exporter_CellExporting(object sender, CellExportingEventArgs e)
        {
            Style cellStyle = e.Field.Settings.CellValuePresenterStyle;
            if (cellStyle == null)
            {
                if (Application.Current.Resources.Contains(typeof(CellValuePresenter)))
                    cellStyle = Application.Current.Resources[typeof(CellValuePresenter)] as Style;
            }

            if (cellStyle != null)
            {
                 Setter bgSetter = null;
                Setter fgSetter = null;
                if (cellStyle.Triggers.Count > 0)
                {
                    DataTrigger bgTrigger = cellStyle.Triggers.Where(tr =>
                        (tr as DataTrigger).Setters.Where(s => s is Setter && (s as Setter).Property.Name.Equals("Background")).FirstOrDefault() != null).FirstOrDefault() as DataTrigger;

                    DataTrigger frTrigger = cellStyle.Triggers.Where(tr =>
                     (tr as DataTrigger).Setters.Where(s => s is Setter && (s as Setter).Property.Name.Equals("Foreground")).FirstOrDefault() != null).FirstOrDefault() as DataTrigger;
                    GreaterThan5Converter conv = new GreaterThan5Converter();

                    //if (bgTrigger != null && conv.Convert(e.Value, typeof(string), null, Thread.CurrentThread.CurrentCulture).ToString().Equals("True"))
                    if (bgTrigger != null && conv.Convert(e.Record.Cells["Open"].Value, typeof(string), null, Thread.CurrentThread.CurrentCulture).ToString().Equals("True"))
                    bgSetter = bgTrigger.Setters.FirstOrDefault() as Setter;
                    if (frTrigger != null && conv.Convert(e.Value, typeof(string), null, Thread.CurrentThread.CurrentCulture).ToString().Equals("True"))
                    fgSetter = frTrigger.Setters.FirstOrDefault() as Setter;
                }
                else
                {
                    bgSetter = cellStyle.Setters.Where(s => s is Setter && (s as Setter).Property.Name.Equals("Background")).FirstOrDefault() as Setter;
                    fgSetter = cellStyle.Setters.Where(s => s is Setter && (s as Setter).Property.Name.Equals("Foreground")).FirstOrDefault() as Setter;

                }
                
                if (bgSetter != null)
                    e.FormatSettings.FillPatternForegroundColor = (bgSetter.Value as SolidColorBrush).Color;


                if (fgSetter != null)
                    e.FormatSettings.FontColor = (fgSetter.Value as SolidColorBrush).Color;
            }
        }

        void exporter_HeaderLabelExporting(object sender, HeaderLabelExportingEventArgs e)
        {
            Style labelStyle = e.Field.Settings.LabelPresenterStyle;
            if (labelStyle == null)
            {
                if (Application.Current.Resources.Contains(typeof(LabelPresenter)))
                    labelStyle = Application.Current.Resources[typeof(LabelPresenter)] as Style;
            }

            if (labelStyle != null)
            {
                Setter bgSetter = labelStyle.Setters.Where(s => s is Setter && (s as Setter).Property.Name.Equals("Background")).FirstOrDefault() as Setter;
                Setter fgSetter = labelStyle.Setters.Where(s => s is Setter && (s as Setter).Property.Name.Equals("Foreground")).FirstOrDefault() as Setter;

                if (bgSetter != null && fgSetter != null)
                {
                    e.FormatSettings.FontColor = (fgSetter.Value as SolidColorBrush).Color;
                    e.FormatSettings.FillPatternForegroundColor = (bgSetter.Value as SolidColorBrush).Color;
                }
            }
        }
    }

    public class Data : INotifyPropertyChanged
    {
        private int open;
        private int close;
        private int high;
        private int low;
        private int value;
        private string label;

        public string Label
        {
            get { return label; }
            set
            {
                label = value;
                PropChanged("Label");
            }
        }

        public int Open
        {
            get { return open; }
            set
            {
                open = value;
                PropChanged("Open");
            }
        }

        public int Close
        {
            get { return close; }
            set
            {
                close = value;
                PropChanged("Close");
            }
        }

        public int High
        {
            get { return high; }
            set
            {
                high = value;
                PropChanged("High");
            }
        }

        public int Low
        {
            get { return low; }
            set
            {
                low = value;
                PropChanged("Low");
            }
        }

        public int Value
        {
            get { return value; }
            set
            {
                this.value = value;
                PropChanged("Value");
            }
        }

        public static ObservableCollection<Data> GetData(int points, int rangeStart, int rangeEnd)
        {
            ObservableCollection<Data> data = new ObservableCollection<Data>();
            Random r = new Random();
            for (int i = 0; i < points; i++)
            {
                data.Add(new Data
                {
                    Label = "Item " + i,
                    Open = r.Next(rangeStart, rangeEnd),
                    Close = r.Next(rangeStart, rangeEnd),
                    High = r.Next(rangeStart, rangeEnd),
                    Low = r.Next(rangeStart, rangeEnd),
                    Value = r.Next(rangeStart, rangeEnd),
                });
            }

            return data;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        public void PropChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
    }
}
